
#import "Controller.h"


static void runOneTick(DPSTimedEntry timedEntry, double timeNow, void *data)
{
    [(id)data ping:timeNow];
	return;
}




@implementation Controller

- appDidInit:sender
{	
	char	*feilname;
	malloc_singlethreaded();


	feilname=malloc(MAXPATHLEN*sizeof(char));
	sprintf(feilname,"%s/.NeXT/ConsoleFilter", NXHomeDirectory());
	matcher=[[Match alloc] initFromFile:feilname];
	free(feilname);
	
	if( NULL== (feilname=(char *)NXGetDefaultValue([NXApp appName],"ConsoleFilename")) )
	{
		feilname=CONSOLE;
		NXWriteDefault([NXApp appName],"ConsoleFilename",feilname);
	}
	
	if(-1==(textFile=open(feilname,O_RDONLY)))
	{
		NXRunAlertPanel("Fatal Error","Can't open console logfile","Quit",NULL,NULL);
		exit(1);
	}
	[window setTitleAsFilename:feilname];
	[window useOptimizedDrawing:YES];
	[window setFrameAutosaveName:"ConsoleView"];
	[window setFrameUsingName:"ConsoleView"];
	[window setAvoidsActivation:YES];
	[text setMonoFont:YES];
	[text setEditable:NO];
	[text setSelectable:YES];
	[text setRetainedWhileDrawing:NO];
	
	
	
	if( NULL==NXGetDefaultValue([NXApp appName],"AutoPopUp") )
	{
		NXWriteDefault([NXApp appName],"AutoPopUp","YES");
	}	
	if(!strcmp("YES",NXGetDefaultValue([NXApp appName],"AutoPopUp")))
		autoPopUp=YES;
	else
		autoPopUp=NO;
	
	[self displayAutoPopUp:self];

	length=0;
	changedLastTime=NO;
	timedEntry=DPSAddTimedEntry(ONE_TICK,&runOneTick,self,NX_BASETHRESHOLD);

	return self;
}

- appDidBecomeActive:sender
{
	[window makeKeyAndOrderFront:sender];
	return self;
}

- appWillTerminate:sender
{
	DPSRemoveTimedEntry(timedEntry);
	close(textFile);
	return self;
}

- showInfoPanel:sender
{
	if( infoPanel==nil && ![NXApp	loadNibSection:"Info.nib" owner:self withNames: NO])
		return nil;
	[infoPanel makeKeyAndOrderFront:nil];
    return self;
}

- showPreferences:sender
{
	if( preferencesPanel==nil )
	{
		if(![NXApp	loadNibSection:"Preferences.nib" owner:self withNames: NO])
			return nil;
		[self revert:self];
		[preferencesPanel setFrameAutosaveName:"FilterPanel"];
		[preferencesPanel setFrameUsingName:"FilterPanel"];
	}

	[autoPopUpFilterButton setState:autoPopUp];
	[preferencesPanel makeKeyAndOrderFront:nil];
    return self;
}

- revert:sender
{
	NXStream	*stream;
	char		*feilname;

	feilname=malloc(MAXPATHLEN*sizeof(char));
	sprintf(feilname,"%s/.NeXT/ConsoleFilter", NXHomeDirectory());
	if(NULL!=(stream=NXMapFile(feilname,NX_READONLY)))
	{
		[preferencesText readText:stream];
		NXClose(stream);
	}
	free(feilname);
    return self;
}

- save:sender
{
	NXStream	*stream;
	char		*feilname;

	feilname=malloc(MAXPATHLEN*sizeof(char));
	sprintf(feilname,"%s/.NeXT/ConsoleFilter", NXHomeDirectory());
	stream=NXOpenMemory(NULL,0,NX_WRITEONLY);
	[preferencesText writeText:stream];
	NXSaveToFile(stream,feilname);
	NXClose(stream);
	[matcher readFromFile:feilname];
	free(feilname);
    return self;
}


- open:sender
{
	char	*feilname;
	id		openPanel=[[OpenPanel new] setTitle:"ConsoleFilename"];
	
	if(! [openPanel runModalForDirectory:NULL file:NULL] ) return nil;
	
	if( NULL!=(feilname=[openPanel filename]) )
	{
		NXWriteDefault([NXApp appName],"ConsoleFilename",feilname);
		close(textFile);
		if(-1==(textFile=open(feilname,O_RDONLY)))
		{
			NXRunAlertPanel("Fatal Error","Can't open console logfile","Quit",NULL,NULL);
			exit(1);
		}
		[window setTitleAsFilename:feilname];
		length=0;
	}
	else
	{
		return nil;
	}
	return self;
}


- ping:(double)tick
{
	static	mom,size;
	static	char *buffer;
	static	matchedLastTime=NO;
	
	if(length!=(mom=lseek(textFile,0,SEEK_END)))
	{
		if(-1==mom)
		{
			NXRunAlertPanel("Fatal Error","Can't seek console logfile","Quit",NULL,NULL);
			DPSRemoveTimedEntry(timedEntry);
			exit(1);
		}
		size=mom-length;
		if(size<0)
		{
			length=0;
			size=mom;
		}
		buffer=(char *)malloc(size);
			lseek(textFile,length,SEEK_SET);
			if(-1==read(textFile,buffer,size))
			{
				NXRunAlertPanel("Fatal Error","Can't read console logfile","Quit",NULL,NULL);
				DPSRemoveTimedEntry(timedEntry);
				exit(1);
			}
			if([matcher matchBuffer:(u_char *)buffer size:size]) matchedLastTime=YES;			
			changedLastTime=YES;
			length+=size;
			[text setSel:[text textLength] :[text textLength]];
			[text replaceSel:(const char*)buffer length:size];
		free(buffer);
	}
	else
	{
		if(changedLastTime)
		{
			[text setSel:[text textLength] :[text textLength]];
			[text scrollSelToVisible];
			changedLastTime=NO;

			if(!matchedLastTime && autoPopUp)
			{
				[window orderWindow:NX_ABOVE relativeTo:0];
				[NXApp activateSelf:NO];
			}
			matchedLastTime=NO;
		}
	}
	return self;
}

- clear:sender
{
	[text setSel:0 :[text textLength]];
	[text replaceSel:(const char*)NULL length:0];
    return self;
}

- reread:sender
{
	[self clear:self];
	length=0;
    return self;
}

- toggleAutoPopUp:sender
{
	autoPopUp= !autoPopUp;

	if(autoPopUp)
		NXWriteDefault([NXApp appName],"AutoPopUp","YES");
	else
		NXWriteDefault([NXApp appName],"AutoPopUp","NO");

	[self displayAutoPopUp:self];
	return self;
}

- displayAutoPopUp:sender
{
	if( autoPopUpMenueButton )
	{
		if(autoPopUp)
			[autoPopUpMenueButton setTitle:"Manual PopUp"];
		else
			[autoPopUpMenueButton setTitle:"Automatic PopUp"];

		[mainMenu sizeToFit];
	}

	if( autoPopUpFilterButton )	[autoPopUpFilterButton setState:autoPopUp];
	return self;
}



- hideAndClear:sender
{
	[NXApp hide:sender];
	[self clear:sender];
    return self;
}


@end
